package game.net;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.logging.*;

import game.Side;
import game.Student;
import game.protocol.*;
import simulator.model.CartesianVect;
import simulator.model.ISimulatorNet;
import simulator.model.PolarVect;

/**
 * Run simulator side of SCORE protocol.
 * @version $Revision: 384 $ $Date: 2008-03-11 09:55:27 -0230 (Tue, 11 Mar 2008) $
 *
 */
public class SCORESimulator implements INetSimulator, Runnable
{
    public SCORESimulator(ISimulatorNet sim)
    {
        mMsgBuffer = new StringBuffer();
        mNetwork   = new Server_network_layer();
        mSimulator = sim;
        mSimulator.setNet(this);
        init();
        startReqListener();
    }

    synchronized public void gameOver(Side result)
    {
        //stopReqListener();
        mReqQueue.flush();
        mState = State.END;
        mResult = result;
        //startReqListener();
    }

    synchronized public void gameReset()
    {
        //stopReqListener();
        mReqQueue.flush();
        mState = State.RESET;
        //startReqListener();
    }

    synchronized public void goalScored(int westScore, int eastScore)
    {
        //stopReqListener();
        mReqQueue.flush();
        mState = State.GOAL;
        mTeams[0].setScore( (mTeams[0].getSide() == Side.WEST) ? westScore : eastScore );
        mTeams[1].setScore( (mTeams[1].getSide() == Side.WEST) ? westScore : eastScore );
        //startReqListener();
    }

    public void run()
    {
        try
        {
            mNetwork.init(mSimulator.getFieldName().getPort());
        }
        catch (UnknownHostException ex)
        {
            // TODO error-handling
        }
        catch (IOException ex)
        {
            // TODO error-handling
        }
        while( !mReqListener.stopping() )
        {
            serveRequests();
        }
        mReqListener = null;

    }

    synchronized public void shutdown() {
    	log.info("Shutting down.");
    	try {
			stopReqListener();
			if (mNetwork != null) mNetwork.shutdown();
		} catch (IOException e) {
			log.fine("Exception in shutdown: " + e.toString());
		}
    	
    }
    
    synchronized public boolean isAlive() {
    	return (mReqListener != null && mReqListener.isAlive());
    }
    /**
     * @param line
     * @throws IOException
     */
    private void assignSide(int line)
            throws IOException, ParseException
    {
        try
        {
            WantSideRequest req = (WantSideRequest) Request.factory( mMsgBuffer.toString() );
            mTeams[line-1].setSide( req.getSide() );
            mTeams[line-1].setHasPickedSide(true);
            if( mTeams[otherLine(line)-1].getHasPickedSide() )
            {
                assignSides(otherLine(line), line);
                mSimulator.startGame();

                OnSideReply reply = new OnSideReply( mTeams[line-1].getSide() );
                sendReply(reply.toString(), line);
                reply = new OnSideReply( mTeams[otherLine(line)-1].getSide() );
                sendReply(reply.toString(), otherLine(line));
                mState = State.NORMAL;
                // reset flags for next time.
                mTeams[line-1].setHasPickedSide(false);
                mTeams[otherLine(line)-1].setHasPickedSide(false);
            }
        }
        catch(ClassCastException ex)
        {
            sendQuit(line);
        }
    }

    private void assignSides(int first, int second)
    {
        if( mTeams[first-1].getSide() == mTeams[second-1].getSide() )
        {
            mTeams[second-1].setSide( (mTeams[second-1].getSide() == Side.WEST) ? Side.EAST : Side.WEST );
        }
        Student west = (mTeams[0].getSide() == Side.WEST) ? mTeams[0].getStudent() : mTeams[1].getStudent();
        Student east = (mTeams[0].getSide() == Side.EAST) ? mTeams[0].getStudent() : mTeams[1].getStudent();
        mSimulator.setTeams(west, east);
    }

    private int getLine()
            throws IOException
    {
        return mNetwork.get_line(Server_network_interface.WEST |
                                 Server_network_interface.EAST,
                                 mMsgBuffer);
    }

    private void handshake(int line)
            throws IOException, ParseException
    {
        try
        {
            SOCCER2008Request req = (SOCCER2008Request) Request.factory( mMsgBuffer.toString() );
            mTeams[line-1].setStudent( req.getStudent() );
            SOCCER2008Reply reply = new SOCCER2008Reply();
            sendReply(reply.toString(), line);
            mTeams[line-1].setHasHandshaked(true);
            if( mTeams[otherLine(line)-1].getHasHandshaked() )
            {
                mState = State.CHOOSINGSIDES;
                mTeams[line-1].setHasHandshaked(false);
                mTeams[otherLine(line)-1].setHasHandshaked(false);
            }
        }
        catch(ClassCastException ex)
        {
            sendEnd(line, true);
        }
    }

    private void init()
    {
        mTeams       = new Team[2];
        mTeams[0] = new Team();
        mTeams[1] = new Team();
        mReqQueue    = new RequestQueue();
        mResult = null;
        mState = State.HANDSHAKING;
    }

    private int otherLine(int line)
    {
        return (line == 1) ? 2 : 1;
    }

    private void sendEnd(int line, boolean scoreError)
           throws IOException, ParseException
   {
       Request req = Request.factory(mMsgBuffer.toString());
//       if (scoreError || req instanceof ControlRequest ||
//           req.getType() == Request.RequestType.ALL ||
//           req.getType() == Request.RequestType.END)
//       {
       if (mState != State.CHOOSINGSIDES && mState != State.HANDSHAKING && mState != State.QUIT) {
    	   mState = State.END;
       }
       if (!mTeams[line-1].getHasSeenEnd() || req.getType() == Request.RequestType.END) {
    	   EndReply reply = new EndReply(mResult);
    	   sendReply(reply.toString(), line);
    	   mTeams[line - 1].setHasSeenEnd(true);      	   
       }
       if (mTeams[otherLine(line) - 1].getHasSeenEnd())
       {
    	   mSimulator.endGame();
    	   mState = State.CHOOSINGSIDES;
    	   mTeams[otherLine(line) - 1].setHasSeenEnd(false);
    	   mTeams[line-1].setHasSeenEnd(false);
    	   mReqQueue.flush();
       }
//     } else {
//    	   log.fine("Invalid state to send end: " + mState);
//       }
//       }
//       else
//       {
//           sendEnd(line, true);
//       }
   }

   private void sendGoal(Request req, int line, int time)
           throws IOException, ParseException
   {
       if (req instanceof ControlRequest ||
           req.getType() == Request.RequestType.ALL)
       {
           if( !mTeams[line-1].getHasSeenGoal() )
           {
               int west = (mTeams[0].getSide() == Side.WEST) ? mTeams[0].getScore() : mTeams[1].getScore();
               int east = (mTeams[0].getSide() == Side.EAST) ? mTeams[0].getScore() : mTeams[1].getScore();
               GoalReply reply = new GoalReply(west, east);
               sendReply(reply.toString(), line);
               mTeams[line-1].setHasSeenGoal(true);
           }
           else
           {
               serveRequest(req, line, time);
           }
           if( mTeams[otherLine(line) - 1].getHasSeenGoal() )
           {
               mState = State.NORMAL;
               mTeams[line-1].setHasSeenGoal(false);
               mTeams[otherLine(line) - 1].setHasSeenGoal(false);
           }
       }
       else
       {
           sendEnd(line, true);
       }
   }


    private void sendQuit(int line)
            throws IOException, ParseException
    {
        Request req = Request.factory( mMsgBuffer.toString() );
        if( req.getType() == Request.RequestType.QUIT ||
            req.getType() == Request.RequestType.WANTSIDE )
        {
            mState = State.QUIT;
            QuitReply reply = new QuitReply();
            sendReply(reply.toString(), line);
            mTeams[line-1].setHasSeenQuit(true);
            if( mTeams[otherLine(line)-1].getSeenQuit() )
            {
                shutdown();
            }
        }
        else
        {
            sendEnd(line, true);
        }
    }

    private void sendReply(String re, int line)
            throws IOException
    {
        mNetwork.send_line(re, line);
    }

    private void sendReset(Request req, int line, int time)
            throws IOException, ParseException
    {
        if (req instanceof ControlRequest ||
            req.getType() == Request.RequestType.ALL)
        {
            if( !mTeams[line-1].getHasSeenReset() )
            {
                ResetReply reply = new ResetReply();
                sendReply(reply.toString(), line);
                mTeams[line-1].setHasSeenReset(true);
            }
            else
            {
                serveRequest(req, line, time);
            }
            if( mTeams[otherLine(line) - 1].getHasSeenReset() )
            {
                mState = State.NORMAL;
                mTeams[line-1].setHasSeenReset(false);
                mTeams[otherLine(line) - 1].setHasSeenReset(false);
            }
        }
        else
        {
            sendEnd(line, true);
        }
    }

    private void serveRequest(Request req, int line, int time)
            throws IOException, ParseException
    {
        if(req.getType() == Request.RequestType.ALL)
        {
        	log.finest("Serving all request.");
            AllReply reply = new AllReply(mSimulator);
            mNetwork.send_line(reply.toString(), line);
        }
        else if(req instanceof ControlRequest)
        {
        	log.finest("Serving control request: " + req.getType());
            mNetwork.send_line(time + "\n", line);

            ControlRequest creq = (ControlRequest) req;
            creq.setSide( mTeams[line-1].getSide() );
            if(creq.getTime() <= time)
            {
                creq.dispatch(mSimulator);
            }
            else
            {
                mReqQueue.addRequest(creq);
            }
        }
        else
        {
        	log.finer("Invalid request type " + req.getType() + ". Ending.");
            sendEnd(line, true);
        }
    }

    private void serveRequests()
    {
        try
        {
            int line = getLine();
            if(line == 0)
            {
                int time = mSimulator.getTime();
                mReqQueue.dispatchReqs(time, mSimulator);
                try {
					Thread.sleep(5);
				} catch (InterruptedException e) { // Ignore execption.
				}
                return;
            }
            log.finest("State = " + mState);
            switch (mState)
            {
                case HANDSHAKING:
                    handshake(line);
                    break;
                case CHOOSINGSIDES:
                    assignSide(line);
                    break;
                case QUIT:
                    sendQuit(line);
                    break;
                case END:
                    sendEnd(line, true);
                    break;
                default:
	                Request req = Request.factory(mMsgBuffer.toString());
	                if (req.getType() == Request.RequestType.END) {
	                	sendEnd(line, false);
	                } else {
	                	int time = mSimulator.getTime();
	                	switch (mState)
	                	{
	                	case RESET:
	                		sendReset(req, line, time);
	                		break;
	                	case GOAL:
	                		sendGoal(req, line, time);
	                		break;
	                	case NORMAL:
	                		serveRequest(req, line, time);
	                		break;
	                	default:
	                		// TODO should never happen
	                		break;
	                	}
	                	//mReqQueue.dispatchReqs(time, mSimulator);
	                }
                    break;
            }
        }
        catch (IOException ex)
        {
            stopReqListener();
        }
        catch (ParseException ex)
        {
            // TODO should never happen
        }
    }

    private void startReqListener()
    {
        mReqListener = new StoppableThread(this);
        mReqListener.start();
    }

    private void stopReqListener()
    {
        if (mReqListener != null) mReqListener.gentleStop(true, 0);
    }

    private enum State { HANDSHAKING, CHOOSINGSIDES, NORMAL, RESET, GOAL, END, QUIT }

    private StringBuffer         mMsgBuffer;
    private Server_network_layer mNetwork;
    private StoppableThread      mReqListener;
    private RequestQueue         mReqQueue;
    private Side                 mResult;
    private ISimulatorNet        mSimulator;
    private State                mState;
    private Team[]               mTeams;
    private static Logger log = Logger.getLogger("Soccer.simulator");
}
