/**
 * SCORETeam.java
 * @author dpeters
 * Created: Mar 6, 2006
 */
package game.net;

import java.io.IOException;
import java.util.logging.*;

import game.Side;
import game.Student;
import game.protocol.*;
import game.protocol.Reply.ReplyType;
import simulator.model.CartesianVect;
import simulator.model.ISimulatorTeam;

/**
 * @version $Revision: 332 $ $Date: 2008-02-07 22:52:19 -0330 (Thu, 07 Feb 2008) $
 */
public class SCORETeam implements ISimulatorTeam {

	public SCORETeam() {
		mNet = new Client_network_layer();
		log.finest("SCORETeam.mState = DISCONNECTED");
		mState = State.DISCONNECTED;
	}
	
	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorTeam#connect(java.lang.String, game.Student, game.Student)
	 */
	public synchronized boolean connect(String host, Student sim, Student team) {
		boolean result = false;
		try {
			mNet.init(host, sim.getPort());
			log.fine("Sending SOCCER2008 " + team);
			mNet.send_line(new SOCCER2008Request(team).toString());
			Reply repl = getReply();
			if (repl.getType() == ReplyType.SOCCER2008) {
				log.info("Connected.");
				log.finest("SCORETeam.mState = INIT");
				mState = State.INIT;
				result = true;
			} else {
				log.warning("Unexpected reply type:" + repl.getType());
			}
		} catch (Exception e) {
			log.severe(e.toString());
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorTeam#initGame(game.Side)
	 */
	public synchronized Reply initGame(Side prefSide) throws ParseException, IOException {
		Reply result = null;
		if (mState == State.INIT) {
			log.fine("Sending want_side " + prefSide);
			mNet.send_line(new WantSideRequest(prefSide).toString());
			result = getReply();
			if (result.getType() == ReplyType.ONSIDE) {
				log.info("Game started.");
				log.finest("SCORETeam.mState = PLAYING");
				mState = State.PLAYING;
			}
		} else { // call in invalid state results in exit.
			log.severe("Protocol violation: SCORETeam.mState != INIT.");
			result = end();
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorTeam#getGameData()
	 */
	public synchronized Reply getGameData() throws IOException, ParseException {
		Reply result = null;
		if (mState == State.PLAYING) {
			log.fine("Sending all request.");
			mNet.send_line(new AllRequest().toString());
			result = getReply();
		} else {
			log.severe("Protocol violation: SCORETeam.mState != PLAYING.");
			result = end();
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorTeam#spin(int, int, double)
	 */
	public synchronized Reply spin(int t, int p, double av) throws IOException, ParseException {
		Reply result = null;
		if (mState == State.PLAYING) {
			log.fine("Sending spin request.");
			try {
				mNet.send_line(new SpinRequest(t, p, av).toString());
				result = getReply();
			} catch (SCOREException e) {
				log.severe("Protocol violation: " + e.toString());
				result = end();
			}
		} else {
			log.severe("Protocol violation: SCORETeam.mState != PLAYING.");
			result = end();
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorTeam#accelerate(int, int, double)
	 */
	public synchronized Reply accelerate(int t, int p, double a) throws IOException, ParseException {
		Reply result = null;
		if (mState == State.PLAYING) {
			log.fine("Sending accelerate request.");
			try {
				mNet.send_line(new AccelerateRequest(t, p, a).toString());
				result = getReply();
			} catch (SCOREException e) {
				log.severe("Protocol violation: " + e.toString());
				result = end();
			}
		} else {
			log.severe("Protocol violation: SCORETeam.mState != PLAYING.");
			result = end();
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorTeam#kick(int, int, double, double)
	 */
	public synchronized Reply kick(int t, int p, double v, double alpha) throws IOException, ParseException {
		Reply result = null;
		if (mState == State.PLAYING) {
			log.fine("Sending spin request.");
			try {
				mNet.send_line(new KickRequest(t, p, v, alpha).toString());
				result = getReply();
			} catch (SCOREException e) {
				log.severe("Protocol violation: " + e.toString());
				result = end();
			}
		} else {
			log.severe("Protocol violation: SCORETeam.mState != PLAYING.");
			result = end();
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorTeam#end()
	 */
	public synchronized Reply end() throws IOException, ParseException {
		Reply result = null;
		log.fine("Requesting end game.");
		mNet.send_line(new EndRequest((Side)null).toString());
		result = getReply();
		if (result.getType() != ReplyType.ENDGAME) {
			log.severe("Protocol volation. Expected end, got " + result.getType());
			result = quit();
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorTeam#quit()
	 */
	public Reply quit() throws IOException, ParseException {
		Reply result = null;
		log.fine("Sending quit");
		if (mState != State.INIT) {
			log.warning("Attempting to quit when protocol not in INIT state, sending anyway.");
		}
		mNet.send_line(new QuitRequest().toString());
		result = getReply();
		if (result.getType() == ReplyType.QUIT) {
			log.info("Clean Quit. Disconnecting.");
		} else {
			log.warning("Invalid reply to Quit: " + result.getType() + " Disconnecting anyway.");
		}
		log.finest("SCORETeam.mState = DISCONNECTED");
		mNet.shutdown();
		mState = State.DISCONNECTED;
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorTeam#place_player(int, int, simulator.model.CartesianVect)
	 */
	public synchronized Reply place_player(int t, int p, CartesianVect l) throws IOException, ParseException {
		Reply result = null;
		if (mState == State.PLAYING) {
			log.fine("Sending place_player request.");
			try {
				mNet.send_line(new PlacePlayerRequest(t, p, l).toString());
				result = getReply();
			} catch (SCOREException e) {
				log.severe("Protocol violation: " + e.toString());
				result = end();
			}
		} else {
			log.severe("Protocol violation: SCORETeam.mState != PLAYING.");
			result = end();
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorTeam#place_ball(int, simulator.model.CartesianVect)
	 */
	public synchronized Reply place_ball(int t, CartesianVect l) throws IOException, ParseException {
		Reply result = null;
		if (mState == State.PLAYING) {
			log.fine("Sending place_ball request.");
			try {
				mNet.send_line(new PlaceBallRequest(t, l).toString());
				result = getReply();
			} catch (SCOREException e) {
				log.severe("Protocol violation: " + e.toString());
				result = end();
			}
		} else {
			log.severe("Protocol violation: SCORETeam.mState != PLAYING.");
			result = end();
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see simulator.model.ISimulatorTeam#set_time(int, int)
	 */
	public synchronized Reply set_time(int t, int newT) throws IOException, ParseException {
		Reply result = null;
		if (mState == State.PLAYING) {
			log.fine("Sending set_time request.");
			try {
				mNet.send_line(new SetTimeRequest(t, newT).toString());
				result = getReply();
			} catch (SCOREException e) {
				log.severe("Protocol violation: " + e.toString());
				result = end();
			}
		} else {
			log.severe("Protocol violation: SCORETeam.mState != PLAYING.");
			result = end();
		}
		return result;
	}
	
	private Reply getReply() throws ParseException, IOException {
		Reply repl = null;
		StringBuffer replTxt = new StringBuffer();
		try {
			while (!mNet.get_line(replTxt)) Thread.sleep(5);
			repl = Reply.factory(replTxt.toString());
			if (repl.getType() == ReplyType.ENDGAME) {
				log.info("Got end reply. Aborting game.");
				log.finest("SCORETeam.mState = INIT");
				mState = State.INIT;
			}
		} catch (InterruptedException e) {
			log.warning("Exception from network: get_line" + e.toString());
		}
		return repl;
	}

	private enum State { DISCONNECTED, INIT, PLAYING }
	
	private static Logger log = Logger.getLogger("Soccer.team");
	private State mState;
	private Client_network_layer mNet;
}
