package game.net ;

import game.net.Buffer;

import java.io.* ;
import java.net.* ;
import java.util.logging.*;


/**
    This class is used by the server to get and send from the network.
    The server should be started first. Up to two client connections
    are accepted. These are referred to as west and east.
*/
public class Server_network_layer
    implements Server_network_interface
{
// PUBLIC PART
    /** Construct a network layer object. */
    public Server_network_layer() { }

    /** Initialize the network layer.
        @param server_port this is the "well known port number" for the server.
        @exception java.net.UnknownHostException if the server host can not be found.
        @exception java.io.IOException if communication with the server can not be initialized.
    */
    public void init( int server_port )
        throws java.net.UnknownHostException,
               java.io.IOException
    {
        if( initialized )
            throw new java.io.IOException("Layer already initialized" ) ;

        server_socket = new ServerSocket( server_port ) ;
        log.finer("Established server socket "+server_socket) ;

        west_socket = server_socket.accept() ;
        log.finer("Accepted one" ) ;

        east_socket = server_socket.accept() ;
        log.finer("Accepted two" ) ;

        west_buffer = new Buffer( west_socket.getInputStream() ) ;
        west_out_stream = west_socket.getOutputStream() ;

        east_buffer = new Buffer( east_socket.getInputStream() ) ;
        east_out_stream = east_socket.getOutputStream() ;

        initialized = true ;
    }

    /** Sends a line to one client.
        @see Server_network_interface
    */
    public void send_line( String line, int which )
        throws java.io.IOException
    {
        if( ! initialized )
            throw new java.io.IOException("Layer not initialized" ) ;

        if( which==WEST ) {
            west_out_stream.write( line.getBytes("ASCII") ) ;
            log.finest("Sending to first:\""+line+"\"") ; }
        else if( which==EAST ) {
            east_out_stream.write( line.getBytes("ASCII") ) ;
            log.finest("Sending to second: \""+line+"\"") ; }
        else throw new  IllegalArgumentException("which must be EAST or WEST") ;
    }

    /** Attempt to read one line from a subset of the clients.
        @see Server_network_interface
    */
    public int get_line( int mask, StringBuffer str_buf )
        throws java.io.IOException
    {
        if( ! initialized )
            throw new java.io.IOException("Layer not initialized" ) ;
        
        west_buffer.fill_with_available_characters() ;
        east_buffer.fill_with_available_characters() ;
        
        boolean can_deliver_west = west_buffer.has_line() && (mask&WEST) != 0 ;
        boolean can_deliver_east = east_buffer.has_line() && (mask&EAST) != 0 ;
        
        if( can_deliver_west && (priority==WEST || !can_deliver_east )) {
            // Deliver to the west
                west_buffer.deliver_line( str_buf ) ;
        		log.finest("Got from first \"" + str_buf + "\"");
                if( can_deliver_east ) priority = EAST ;
                return WEST ; }
        else if( can_deliver_east && (priority==EAST || !can_deliver_west)) {
            // Deliver to the east
                east_buffer.deliver_line( str_buf ) ;
    			log.finest("Got from second \"" + str_buf + "\"");
                if( can_deliver_west ) priority = WEST ;
                return EAST ; }
        else {
            return 0 ; }
    }

    /** Finish everything up
    @exception java.io.IOException if any trouble ;
    */
    public void shutdown()
        throws java.io.IOException
    {
        if( ! initialized )
            throw new java.io.IOException("Layer not initialized" ) ;
        log.finer("Closing first.");
        west_buffer.shutdown() ;
        west_out_stream.close() ;
        west_socket.close() ;
        
        log.finer("Closing second.");
        east_buffer.shutdown() ;
        east_out_stream.close() ;
        east_socket.close() ; 
        
        server_socket.close() ; }
        
    /** @deprecated Use
      a_soccertools.interfaces.Server_network_interface.WEST
      instead.
        @see Server_network_interface
      */
    public static final int WEST = Server_network_interface.WEST ;
    /** @deprecated Use
      a_soccertools.interfaces.Server_network_interface.EAST
      instead.
        @see Server_network_interface
      */
    public static final int EAST = Server_network_interface.EAST ;
    
// PRIVATE PART

    private ServerSocket server_socket ;
    private Socket west_socket, east_socket ;
    private Buffer west_buffer, east_buffer ;
    private OutputStream west_out_stream, east_out_stream ;

    private boolean initialized = false ;
    private int priority = WEST ;
    private static Logger log = Logger.getLogger("Soccer.simulator");
    
}