/**
 * AllReply.java
 * @author dennis
 * Created: Feb 21, 2006
 */
package game.protocol;

import game.Side;
import simulator.model.*;


/**
 * Normal reply from simulator to "all?" request. Gives pretty much the whole
 * game state.
 * <p>Invariant: mType == ALL</p>
 */
public class AllReply extends Reply {
	
	/** Parse a normal reply 
	 * @param msg the message text
	 * @throws ParseException ill formatted message
	 */
	public AllReply(String msg) throws ParseException {
		create();
		int[] tokens = new int[2]; // start and end of tokens
		mType = ReplyType.ALL;
		try {
			tokens[0] = 0;
			tokens[1] = msg.indexOf(" ");
			msgTime = new Integer(msg.substring(0, tokens[1]));
			double x;
			double y;
			double omega;
			double v;
			x = nextDouble(msg, tokens);
			y = nextDouble(msg, tokens);
			ballPos = SMVector.makeCartesian(x, y);
			v = nextDouble(msg, tokens);
			omega = nextDouble(msg, tokens);
			ballVel = SMVector.makePolar(v, omega);
			for (int t = 0; t < 2; t++) {
				for (int p = 0; p < ISimulatorView.NUMBER_OF_PLAYERS; p++) {
					tokens[0] = tokens[1]+1;
					tokens[1] = msg.indexOf(" ", tokens[0]);
					inGame[t][p] = (new Integer(msg.substring(tokens[0], tokens[1])) == 1);
					x = nextDouble(msg, tokens);
					y = nextDouble(msg, tokens);
					playerPos[t][p] = SMVector.makeCartesian(x, y);
					v = nextDouble(msg, tokens);
					omega = nextDouble(msg, tokens);
					playerVel[t][p] = SMVector.makePolar(v, omega);
				}
			}
			log.fine("Got good all reply. msgTime = " + msgTime);
		} catch (NumberFormatException e) {
			ParseException pe = new ParseException("Expected number", msg, tokens[0], e);
			log.throwing("AllReply", "AllReply(String)", pe);
			throw pe;
		}
	}
	
	/**
	 * A bit different in that this constructor will extract the required info
	 * from the given sim.
	 * 
	 * @param sim model to draw data from.
	 */
	public AllReply(ISimulatorView sim) {
		// TODO catch invalid states, throw exception and log it.
		create();
		mType = ReplyType.ALL;
		msgTime = sim.getTime();
		ballPos = sim.getBallPosition();
		ballVel = sim.getBallVelocity();
		for (Side t : Side.values()) {
			for (int p = 0; p < ISimulatorView.NUMBER_OF_PLAYERS; p++) {
				inGame[t.ordinal()][p] = sim.isPlayerInGame(t, p);
				playerPos[t.ordinal()][p] = sim.getPlayerPosition(t, p);
				playerVel[t.ordinal()][p] = sim.getPlayerVelocity(t, p);
			}
		}
		
	}
	
	/**
	 * Parse a Double from txt 
	 * @param txt the full message
	 * @param start in: start of previous double, 
	 *              out: updated to indicate the start of this double
	 * @param end  in: end+1 of previous double
	 *            out: updated to indicate the end+1 of this double
	 * @return the parsed double
	 * @throws NumberFormatException if the text isn't a number
	 */
	private Double nextDouble(String txt, int[] indx) throws NumberFormatException {
		indx[0] = indx[1]+1;
		indx[1] = txt.indexOf(" ", indx[0]);
		if (indx[1] == -1) indx[1] = txt.length();
		return new Double(txt.substring(indx[0], indx[1]));		
	}
	
	/**
	 * Construct the arrays etc.
	 */
	private void create() {
		inGame = new boolean[2][ISimulatorView.NUMBER_OF_PLAYERS];
		playerPos = new CartesianVect[2][ISimulatorView.NUMBER_OF_PLAYERS];
		playerVel = new PolarVect[2][ISimulatorView.NUMBER_OF_PLAYERS];
	}
	
	public String toString() {
		String result = new String();
		result += (msgTime + " ");
		result += String.format("%03.3f %03.3f ", ballPos.getX(), ballPos.getY());
		result += String.format("%03.3f %03.3f", ballVel.getMagnitude(), ballVel.getAngle());
		for (int t = 0; t < 2; t++) {
			for (int p = 0; p < ISimulatorView.NUMBER_OF_PLAYERS; p++) {
				result += inGame[t][p] ? " 1 " : " 0 ";
				result += String.format("%03.3f %03.3f", playerPos[t][p].getX(), playerPos[t][p].getY());
				result += String.format(" %03.3f %03.3f", playerVel[t][p].getMagnitude(), playerVel[t][p].getAngle());
			}
		}
		result += "\n";
		return result;
	}
	
	/**
	 * @return time in milliseconds from message
	 */
	public int getTime() {
		return msgTime;
	}
		
    /**
     * @return Ball's current position in meters
     */
    public CartesianVect getBallPosition() {
    	return ballPos;
    }

    /**
     * @return Ball's current velocity in polar coordinates
     */
    public PolarVect getBallVelocity() {
       	return ballVel;
    }

    /**
     * @param t Team
     * @param p Player number in {0, 1, 2, 3, 4}
     * @return Player's current position in meters
     */
    public CartesianVect getPlayerPosition(Side t, int p) {
    	return playerPos[t.ordinal()][p];
    }

    /**
     * @param t Team
     * @param p Player number in {0, 1, 2, 3, 4}
     * @return Player's current velocity in polar coordinates
     */
    public PolarVect getPlayerVelocity(Side t, int p) {
    	return playerVel[t.ordinal()][p];
    }

    /**
     * @param t Team
     * @param p Player number in {0, 1, 2, 3, 4}
     * @return False if player is red-carded and true otherwise
     */
    public boolean isPlayerInGame(Side t, int p) {
    	return inGame[t.ordinal()][p];
    }
	
	/** Timestamp */
	private int msgTime;
	
	private CartesianVect ballPos;
	
	private PolarVect ballVel;
	
	private boolean[][] inGame;
	
	private CartesianVect[][] playerPos;
	
	private PolarVect[][] playerVel;
	

}
