/**
 * MainUI.java
 * @author dpeters
 * Created: Feb. 21, 2006
 */
package simulator;

import java.util.*;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import java.awt.*;
import java.awt.event.*;
import java.io.IOException;

import javax.swing.*;

import game.Side;
import game.Student;
import simulator.model.*;
import simulator.view.*;

/**
 * Top level UI for running simulators.
 */
public class MainUI extends JFrame implements Runnable, ActionListener {

	/**
	 * Note that no UI components are created here. @see createAndShowUI
	 * @throws HeadlessException
	 */
	public MainUI() throws HeadlessException {
	    mUIReady = false;
	    javax.swing.SwingUtilities.invokeLater(this);

	}
	/**
	 * @return true if UI has been created.
	 */
	public boolean uiIsReady() {
		return mUIReady;
	}
	
	/**
	 * Construct the UI. This approach is used to avoid thread safety problems.
	 * @see http://java.sun.com/docs/books/tutorial/uiswing/misc/threads.html
	 */
	private void createAndShowUI() {
	    setTitle("ENGI 9874 SOCCER Simulator Setup");
	    setSize(WIDTH, HEIGHT);
	    Container pane = getContentPane();
	    GridBagLayout gridBag = new GridBagLayout();
		pane.setLayout(gridBag);
	    GridBagConstraints gbConst = new GridBagConstraints();
	    
		JLabel viewChooserLabel = new JLabel("View: ");
		gbConst.weightx = 1.0;
		gbConst.fill = GridBagConstraints.BOTH;
		gbConst.gridwidth = 1;
		gridBag.setConstraints(viewChooserLabel, gbConst);
		pane.add(viewChooserLabel);
		
		viewChooser = new JComboBox(availableViewImplementations());
		gbConst.weightx = 3.0;
		gbConst.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(viewChooser, gbConst);
		pane.add(viewChooser);
		
		JLabel modelChooserLabel = new JLabel("Model: ");
		gbConst.weightx= 1.0;
		gbConst.fill = GridBagConstraints.BOTH;
		gbConst.gridwidth = 1;
		gridBag.setConstraints(modelChooserLabel, gbConst);
		pane.add(modelChooserLabel);
		
		modelChooser = new JComboBox(availableModelImplementations());
		gbConst.weightx = 3.0;
		gbConst.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(modelChooser, gbConst);
		pane.add(modelChooser);
		
		JLabel testModeLabel = new JLabel("Test mode:");
		gbConst.weightx= 1.0;
		gbConst.fill = GridBagConstraints.BOTH;
		gbConst.gridwidth = 1;
		gridBag.setConstraints(testModeLabel, gbConst);
		pane.add(testModeLabel);
		
		testMode = new JCheckBox();
		gbConst.weightx = 3.0;
		gbConst.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(testMode, gbConst);
		pane.add(testMode);
		
	    goButton = new JButton("Go");
	    goButton.addActionListener(this);
		gbConst.weightx = 2.0;
		gbConst.gridwidth = 2; //GridBagConstraints.RELATIVE;
		gridBag.setConstraints(goButton, gbConst);
	    pane.add(goButton);
	    
	    stopButton = new JButton("Stop");
	    stopButton.setEnabled(false);
	    stopButton.addActionListener(this);
		gbConst.weightx = 2.0;
		gbConst.gridwidth = 2;//GridBagConstraints.REMAINDER;
	    gridBag.setConstraints(stopButton, gbConst);
	    pane.add(stopButton);

	    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    setVisible(true);
		mUIReady = true; // must be last line of method 
	}
	
	/** Get a list of available view implementations.
	 * @return vector containing Students who have view implementations available.
	 */
	private Vector<Student> availableViewImplementations() {
		Vector<Student> availViews = new Vector<Student>();
		for (Student s : Student.values()) {
			try {
				Class.forName(viewName(s));
				availViews.add(s);
			} catch (ClassNotFoundException e) { }
		}
		return availViews;
	}
	
	/** Get a list of available model implementations.
	 * @return vector containing Students who have view implementations available.
	 */
	private Vector<Student> availableModelImplementations() {
		Vector<Student> availModels = new Vector<Student>();
		for (Student s : Student.values()) {
			try {
				Class.forName(modelName(s));
				availModels.add(s);
			} catch (ClassNotFoundException e) { }
		}
		return availModels;
	}
	
	/**
	 * Construct the name of the SMfacade class for a given student.
	 * @param s student
	 * @return a fully qualified class name
	 */
	private String modelName(Student s) {
		return new String("simulator.model." + s + ".SMfacade");
	}
	
	/**
	 * Construct the name of the SVmain class for a given student.
	 * @param s student
	 * @return a fully qualified class name
	 */
	private String viewName(Student s) {
		return new String("simulator.view." + s + ".SVmain");
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		createAndShowUI();
	}
	
	/**
	 * Process events from UI.
	 * @param e event
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == goButton) {
			try {
				constructSim();
				simView.start();
				setButtons();
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		} else if (e.getSource() == stopButton) {
			simView.stop();
			mNet.shutdown();
//			TODO stop mNet, simModel.
			setButtons();
		}
		
	}

	/** 
	 * Construct the simulator model and view in preparation to start the game.
	 * @throws InstantiationException
	 * @throws ClassNotFoundException if there is no implementation avialable.
	 * @throws IllegalAccessException
	 */
	private void constructSim() 
		throws InstantiationException, ClassNotFoundException, IllegalAccessException {
		Student m = (Student)modelChooser.getSelectedItem();
		if (m == null) throw new ClassNotFoundException();
		simModel = (simulator.model.ISimulatorView)Class.forName(modelName(m)).newInstance();
		simModel.setTestMode(testMode.isSelected());
		
		Student v = (Student)viewChooser.getSelectedItem();
		if (v == null) throw new ClassNotFoundException();
		simView = (SVbase)Class.forName(viewName(v)).newInstance();
		simView.setModel(simModel);	
		mNet = new game.net.SCORESimulator(simModel.getISimulatorNet());
		svListener = new ViewCloseListener(this);
		simView.addWindowListener(svListener);
	}
	
	/**
	 * Enable and disable buttons as appropriate for game state.
	 */
	private void setButtons() {
		if (svListener != null && svListener.viewExists()) {
			goButton.setEnabled(false);
			stopButton.setEnabled(true);
			testMode.setEnabled(false);
		} else {
			goButton.setEnabled(true);
			stopButton.setEnabled(false);
			testMode.setEnabled(true);
		}
		
	}
	
	public void paint(Graphics g) {
//		if (!mNet.isAlive()) simView.stop();
		setButtons();
		super.paint(g);
	}
	
	/** Indicates if UI has been constructed */
	private boolean mUIReady;

	private JButton goButton;
	private JButton stopButton;
	
	private JComboBox modelChooser;
	private JComboBox viewChooser;
	
	private JCheckBox testMode;

	/** Initial width of frame. */
	private static final int WIDTH = 410;
	
	/** Initial height of frame. */
	private static final int HEIGHT = 200;

	private ISimulatorView simModel;
	
	private SVbase simView;
	
	private ViewCloseListener svListener;

	private game.net.SCORESimulator mNet;

	private static Logger log;

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		log = Logger.getLogger("Soccer.simulator");
		try {
			System.out.println("Log file will be written to: " + 
					System.getProperty("user.home"));
			Handler fh = new FileHandler("%h/soccer_simulator.log", true);
			fh.setFormatter(new SimpleFormatter());
			log.addHandler(fh);
			log.setLevel(Level.FINER); // could be set by UI
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		log.fine("Constructing new simulator.MainUI()");
		MainUI ui = new MainUI();
/*
		try {
			while (!ui.uiIsReady()) Thread.sleep(500);
			
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
*/
	}

}
