package simulator.model;

import game.*;

/**
 * <p>The interface of a simulator as seen by the network layer</p>
 *
 * @author pouria
 * @version 1.0
 */
public interface ISimulatorNet extends ISimulatorView
{

	/** 
	 * Indicates which team is on each side.
	 * @param west team on WEST side.
	 * @param east team on EAST side.
	 */
	public void setTeams(Student west, Student east);
	
	/**
	 * Indicates that game should start (time == 0).
	 * <p>Precondidtion: setTeams and setNet have been called and 
	 * isGameInProgress() is false.<br>
	 * Postcondition: isGameInProgress()</p> 
	 */
	public void startGame();
	
	/**
	 * Indicates game should end due to a request from one or more teams.
	 * Note that the protocol says that this should only occur due to abnormal 
	 * situations. 
	 * <p>Precondition: true<br>
	 * Postcondition: !isGameInProgress()</p> 
	 */
	public void endGame();
	
	/** 
	 * Set the network layer. To be used to call goalScored(), 
	 * gameReset() and gameOver() when these events occur. 
	 * @param n
	 */
	public void setNet(game.net.INetSimulator n);
 
    /**
     * Sets angular velocity for the player
     *
     * <p>Precondition: isGameInProgress()</p>
     * 
     * @param p Player number in {0, 1, 2, 3, 4}
     * @param av double Angular velocity in degrees per second. -90 <= av <= 90
     */
    public void spin(Side side, int p, double av);

    /**
     * Sets the acceleration for the player
     *
     * <p>Precondition: isGameInProgress()</p>
     * @param p Player number, in {0, 1, 2, 3, 4}
     * @param a rate of acceleration, -5 <= a <= 1
     */
    public void accelerate(Side side, int p, double a);

    /**
     * Updates field data in response to a kick request
     *
     * <p>Precondition: isGameInProgress()</p>
     * @param side the team side
     * @param p Player number, in {0, 1, 2, 3, 4}
     * @param v Change in the ball's velocity. 0 <= v <= 10
     * @param alpha Offset in degrees from a line connecting the player's
     *        centre to the ball's centre. -45 <= alpha <= 45
     */
    public void kick(Side side, int p, double v, double alpha);

    /**
     * places player at given coordinates
     *
     * <p>Precondition: game is in test mode</p>
     * @param side Player team
     * @param p Player number. In {0, 1, 2, 3, 4}
     * @param l new location for player
     */
    public void placePlayer(Side side, int p, CartesianVect l);

    /**
     * placeBall places ball at given coordinates
     *
     * <p>Precondition: game is in test mode</p>
     * @param l location to place the ball
     */
    public void placeBall(CartesianVect l);

    /**
     * setTime sets game time
     *
     * <p>Precondition: game is in test mode</p>
     * @param t int Game time is set to t (in miliseconds)
     */
    public void setTime(int t);
}
