package simulator.model;

import java.io.IOException;

import game.protocol.ParseException;
import game.protocol.Reply;
import game.Side;
import game.Student;


/**
 *
 * <p>The interface of a simulator as seen by a Team. Note that deviations 
 * from the pre-conditions/parameter constraints specified for each method 
 * will result in a protocol error and therefore ending of the game.</p>
 *
 * @author pouria
 * @version 1.0
 */
public interface ISimulatorTeam
{
	
	/**
	 * Connect to host using Soccer2006 protocol.
	 * @param host hostname or ipaddress for simulator
	 * @param sim which simulator model will host the game
	 * @param team which team am I
	 * @return true if connection was successful, false otherwise
	 */
	public boolean connect(String host, Student sim, Student team);
	
	/**
	 * Protocol sequence to initialize game.
	 * @param prefSide the preferred side for this team.
	 * @return OnSideReply or QuitReply.
	 * @throws ParseException incorrectly formatted message.
	 * @throws IOException connection/network error
	 */
	public Reply initGame(Side prefSide) throws ParseException, IOException;
	
    /**
     * Send all feedback request.
     * @return normally AllReply. 
     *         GoalReply, ResetReply and EndReply are also possible.
     * @throws IOException connection/network error
     * @throws ParseException incorrectly formatted message.
     */
    public Reply getGameData() throws IOException, ParseException;

    /**
     * Send spin control request. Sets angular velocity for the player
     *
     * @param t Time at which this command is to be executed. 0 &lt;= t
     * @param p Player number. In {0, 1, 2, 3, 4}
     * @param av Angular velocity in degrees per second. -90 &lt;= av &lt;= 90
     * @return normally ControlReply. 
     *         GoalReply, ResetReply and EndReply are also possible.
     * @throws IOException connection/network error
     * @throws ParseException incorrectly formatted message.
     */
    public Reply spin(int t, int p, double av) throws IOException, ParseException;

    /**
     * Send accelerate control request. Sets the acceleration for the player
     *
     * @param t Time at which this command is to be executed. 0 &lt;= t
     * @param p Player number. In {0, 1, 2, 3, 4}
     * @param a rate of acceleration. -5 &lt;= a &lt;= 1
     * @return normally ControlReply. 
     *         GoalReply, ResetReply and EndReply are also possible.
     * @throws IOException connection/network error
     * @throws ParseException incorrectly formatted message.
     */
    public Reply accelerate(int t, int p, double a) throws IOException, ParseException;

    /**
     * Send kick control request. Attempt to kick the ball.
     * 
     * @param t Time at which this command is to be executed. 0 &lt;= t
     * @param p Player number. In {0, 1, 2, 3, 4}
     * @param v Change in the ball's velocity. 0 &lt;= v &lt;= 10
     * @param alpha Offset in degrees from a line connecting the player's
     *        centre to the ball's centre. -45 &lt;= alpha &lt;= 45
     * @return normally ControlReply. 
     *         GoalReply, ResetReply and EndReply are also possible.
     * @throws IOException connection/network error
     * @throws ParseException incorrectly formatted message.
     */
    public Reply kick(int t, int p, double v, double alpha) throws IOException, ParseException;
    
    /**
     * Instruct simulator to abort the game. Normally only in the case 
     * of protocol error.
     * @return EndReply
     * @throws IOException connection/network error
     * @throws ParseException incorrectly formatted message.
     */
    public Reply end() throws IOException, ParseException;
    
    /**
     * Instruct the simulator to disconnect and shut down.
     * @return normally QuitReply
     * @throws IOException connection/network error
     * @throws ParseException incorrectly formatted message.
     */
    public Reply quit() throws IOException, ParseException;
    
    /**
     * Send place_player control request. Sets the position for the player
     *
     * <p>Precondition: Simulator is in Test mode.</p>
     * 
     * @param t Time at which this command is to be executed. 0 &lt;= t
     * @param p Player number. In {0, 1, 2, 3, 4}
     * @param l New position for player
     * @return normally ControlReply. 
     *         GoalReply, ResetReply and EndReply are also possible.
     * @throws IOException connection/network error
     * @throws ParseException incorrectly formatted message.
     */
    public Reply place_player(int t, int p, CartesianVect l) throws IOException, ParseException;
    
    /**
     * Send place_ball control request. Sets the position for the ball
     *
     * <p>Precondition: Simulator is in Test mode.</p>
     * 
     * @param t Time at which this command is to be executed. 0 &lt;= t
     * @param l New position for ball
     * @return normally ControlReply. 
     *         GoalReply, ResetReply and EndReply are also possible.
     * @throws IOException connection/network error
     * @throws ParseException incorrectly formatted message.
     */
    public Reply place_ball(int t, CartesianVect l) throws IOException, ParseException;
    
    /**
     * Send set_time control request. Sets the game time
     *
     * <p>Precondition: Simulator is in Test mode.</p>
     * 
     * @param t Time at which this command is to be executed. 0 &lt;= t
     * @param newT New time
     * @return normally ControlReply. 
     *         GoalReply, ResetReply and EndReply are also possible.
     * @throws IOException connection/network error
     * @throws ParseException incorrectly formatted message.
     */
    public Reply set_time(int t, int newT) throws IOException, ParseException;
}
