package simulator.model;

import game.Side;
import game.Student;

/**
 * <p>An interface used to represent the game state to the simulator view
 * object(s).</p>
 * <p> An acceptable simulator model implementation must contain a class called SMfacade
 *   that
 *   <ol>
 *   <li>implements this interface and</li>
 *   <li>has a no-argument constructor <code>public SMfacade()</code>.</li>
 *   </ol>
 * </p>
 * @version 5.0
*/
public interface ISimulatorView
{
	// Constants
    /** The length (x-axis) of the field in metres. */
    public static final double FIELD_LENGTH = 50;

    /** The width (y-axis) of the field in metres. */
    public static final double FIELD_WIDTH = 25;

    /** The number of players per team */
    public static final int NUMBER_OF_PLAYERS = 5;

    /** The width of the goal. The goals are centred at either end of the field
    */
    public static final double GOAL_WIDTH = 10;

    /** The radius of the ball in metres. */
    public static final double BALL_RADIUS = 0.15;

    /** The radius of each player in metres. */
    public static final double PLAYER_RADIUS = 0.25;


// These functions may be called at any time.
    /**
     * Get the object that implements the ISimulatorNet interface.
     * This isn't needed by the view so strictly it should be somewhere else.
     * It is needed by simulator.MainUI to set up the game.
     *
     * @return an object that is instanceof ISimulatorNet
     */
    public ISimulatorNet getISimulatorNet();

    /**
     * Set the model into 'test mode' or not.
     * @param m true to set the model in test mode, false for normal play mode
     */
    public void setTestMode(boolean m);

    /** Which simulator is the host. */
    public Student getFieldName() ;

    /** The last status message.
    * This could be any string at all.
    */
    public String getStatusMessage( ) ;

    /**
     * @return true if the game is in progress, false otherwise.
     */
    public boolean isGameInProgress();

    /** The name of each team.
    <P> Precondition: isGameInProgress()
    @param t The team, 0 or 1.
    */
    public Student getTeamName( Side t ) ;

    /** The number of goals scored by each team.
    <P> Precondition: isGameInProgress()</p>
    @param t The team, 0 or 1.
    */
    public int getTeamScore( Side t ) ;

    /** The number of milliseconds played.
    <P> Precondition: isGameInProgress()</p>
    */
    public int getTime( ) ;

    /** The position of a particular player.
    <P> Precondition: isGameInProgress()</p>
    @param t The team, 0 or 1.
    @param p The player: 0 less than or equal p, p less than number_of_players()
    */
    public CartesianVect getPlayerPosition(Side t, int p) ;

    /** The orientation of a particular player.
    The orientation is reported in degrees from the x-axis.
    <P> Precondition: isGameInProgress()
    @param t The team, 0 or 1.
    @param p The player: 0 less than or equal p, p less than NUMBER_OF_PLAYERS()
    */
    public PolarVect getPlayerVelocity(Side t, int p) ;

    /** Is the player in play
     * @param t team side
     * @param p player. 0 less than or equal p, p less than number_of_players()
     * @return true if the player is in play, false otherwise (i.e., red-carded).
     */
    public boolean isPlayerInGame(Side t, int p);

    /**
     * @return The position of the ball.
     */
    public CartesianVect getBallPosition() ;

    /**
     * @return The velocity of the ball.
     */
    public PolarVect getBallVelocity();
}
