/**
 * SMVector.java
 */
package simulator.model;

/** Vector manipulation class that implements both PolarVect and 
 * CartesianVect interfaces.
 *  
 * @version $Revision: 336 $ $Date: 2008-02-10 22:19:15 -0330 (Sun, 10 Feb 2008) $
 */
public class SMVector implements PolarVect, CartesianVect {

	private SMVector() {
		// disallow no-argument construction
	}
	
	/**
	 * Construct a vector from polar coordinates
	 * @param mag magnitude 0 <= mag
	 * @param ang angle in degrees from the positive x axis. 0 <= ang < 360.
	 * @return new vector
	 */
	static public SMVector makePolar(double mag, double ang) {
		SMVector v = new SMVector();
		assert (mag >= 0.0);
		assert (0 <= ang && ang < 360);
		v.mag = mag;
		v.ang = ang;
		v.form = Form.POLAR;

		return v;
	}
	
	/**
	 * Construct a vector from cartesian coordinates
	 * @param x horizontal component 0 is to the left.
	 * @param y vertical component 0 is to the bottom
	 * @return New vector
	 */
	static public SMVector makeCartesian(double x, double y) {
		SMVector v = new SMVector();
		v.x = x;
		v.y = y;
		v.form = Form.CARTESIAN;
		return v;
	}

	/* (non-Javadoc)
	 * @see simulator.model.PolarVect#getAngle()
	 */
	public double getAngle() {
		if (form == Form.CARTESIAN) {
			cartesianToPolar();
		}
		return ang;
	}

	/* (non-Javadoc)
	 * @see simulator.model.PolarVect#getMagnitude()
	 */
	public double getMagnitude() {
		if (form == Form.CARTESIAN) {
			cartesianToPolar();
		}
		return mag;
	}

	/* (non-Javadoc)
	 * @see simulator.model.CartesianVect#getX()
	 */
	public double getX() {
		if (form == Form.POLAR) {
			polarToCartesian();
		}
		return x;
	}

	/* (non-Javadoc)
	 * @see simulator.model.CartesianVect#getY()
	 */
	public double getY() {
		if (form == Form.POLAR) {
			polarToCartesian();
		}
		return y;
	}
	
	/** Set the value of this in Cartesian form
	 * @param x horizontal coord. 0 is to the left
	 * @param y vertical coord. 0 is to the top
	 */
	public void setCartesian(double x, double y) {
		form = Form.CARTESIAN;
		this.x = x;
		this.y = y;
	}
	
	/**
	 * Set the value of this in Polar form
	 * @param mag magnitude. 0 <= mag
	 * @param ang angle in degrees from positive x axis. 0 <= ang < 360 
	 */
	public void setPolar(double mag, double ang) {
		form = Form.POLAR;
		assert (mag >= 0);
		this.mag = mag;
		// force ang to right range
		while (ang < 0) ang += 360;
		while (ang >= 360) ang -= 360;
		this.ang = ang;
	}
	
	/** Update (mag, ang) to match (x, y) */
	private void cartesianToPolar() {
		mag = Math.sqrt(x*x + y*y);
		ang = Math.toDegrees(Math.atan2(y, x)); 
		if (ang < 0) ang += 360;
		form = Form.BOTH;
	}

	/** Update (x, y) to match (mag, ang) */
	private void polarToCartesian() {
		double radAng = Math.toRadians(ang); 
		x = mag*Math.cos(radAng);
		y = mag*Math.sin(radAng);
		form = Form.BOTH;
	}
	
	private enum Form { POLAR, CARTESIAN, BOTH };
	private Form form;
	private double x;
	private double y;
	private double mag;
	private double ang;
}
