/**
 * SVbase.java
 * @author dpeters
 * Created: Feb 15, 2006
 */
package simulator.view;

import java.awt.HeadlessException;
import javax.swing.*;

import simulator.model.ISimulatorView;

/** Base class for the main simulator view class. An acceptable simulator 
 * view implementation must contain the 'main' class called SVmain that
 * <ol>
 * <li>Extends this class (and therefore implements the abstract methods)</li>
 * <li>Has a no argument constructor <code>public SVmain()</code>
 * </ol>
 * 
 */
abstract public class SVbase extends JFrame implements Runnable {

	/** Construct all components of the UI and make them visible. Note that this
	 * is different from the constructor, which should not create the UI components
	 * for thread safety reasons. The last line of this method should set mUIReady to true.
	 */
	abstract protected void createAndShowUI();

	/** Set the model for this view.
	 * @param model the simulator model that is hosting the game.
	 */
	abstract public void setModel(ISimulatorView model);
	
	/** Constructor. Note that UI components shouldn't be created by
	 * this constructor or its extensions -- these are created by createAndShowUI().
	 * You may (and probably should) extend this constructor. The first line of the 
	 * extension constructor should be super() to call this constructor.
	 *  
	 * @throws HeadlessException if the JVM has no graphical capability.
	 */
	public SVbase() throws HeadlessException {
	    mUIReady = false;
	}

	/** Has the UI been constructed yet?
	 * @return true if UI has been created.
	 */
	public boolean uiIsReady() {
		return mUIReady;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		createAndShowUI();
	}
	
	/** 
	 * Causes the view to be created and start running in a thread safe manner.
	 */
	public void start() {
		javax.swing.SwingUtilities.invokeLater(this);
	}
	
	/** 
	 * Shutdown this view. You may extend this if necessary.
	 */
	public void stop() {
		this.dispose();
	}
	
	/** True iff UI has been constructed */
	protected boolean mUIReady;
}
