/**
 * MainUI.java
 * @author dpeters
 * 
 */
package team;

import java.io.IOException;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.util.logging.*;

import javax.swing.*;

import game.Side;
import game.Student;
import game.protocol.*;


/**
 * Top level UI for running teams.
 * @version $Revision: 362 $ $Date: 2008-02-19 15:14:02 -0330 (Tue, 19 Feb 2008) $
 */
public class MainUI extends JFrame implements Runnable, ActionListener, ITeamObserver {

	public MainUI() throws HeadlessException {
	    mUIReady = false;
	    javax.swing.SwingUtilities.invokeLater(this);

	}
	/**
	 * @return true if UI has been created.
	 */
	public boolean uiIsReady() {
		return mUIReady;
	}
	
	/**
	 * Construct the UI. This approach is used to avoid thread safety problems.
	 * @see http://java.sun.com/docs/books/tutorial/uiswing/misc/threads.html
	 */
	private void createAndShowUI() {
		log.entering("team.MainUI", "createAndShowUI");
	    setTitle("ENGI 9874 SOCCER Team Setup");
	    setSize(WIDTH, HEIGHT);
	    Container pane = getContentPane();
	    GridBagLayout gridBag = new GridBagLayout();
		pane.setLayout(gridBag);
	    GridBagConstraints gbConst = new GridBagConstraints();
	    
		JLabel teamChooserLabel = new JLabel("Team: ");
		gbConst.weightx = 1.0;
		gbConst.fill = GridBagConstraints.BOTH;
		gbConst.gridwidth = 1;
		gridBag.setConstraints(teamChooserLabel, gbConst);
		pane.add(teamChooserLabel);
		
		teamChooser = new JComboBox(availableTeamImplementations());
		gbConst.weightx = 3.0;
		gbConst.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(teamChooser, gbConst);
		pane.add(teamChooser);
		
		JLabel simHostLabel = new JLabel("Simulator Host: ");
		gbConst.weightx= 1.0;
		gbConst.fill = GridBagConstraints.BOTH;
		gbConst.gridwidth = 1;
		gridBag.setConstraints(simHostLabel, gbConst);
		pane.add(simHostLabel);
		
		simHost = new JTextField(30);
		simHost.setText("localhost");
		gbConst.weightx = 3.0;
		gbConst.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(simHost, gbConst);
		pane.add(simHost);
		
		JLabel simChooserLabel = new JLabel("Simulator Model: ");
		gbConst.weightx= 1.0;
		gbConst.fill = GridBagConstraints.BOTH;
		gbConst.gridwidth = 1;
		gridBag.setConstraints(simChooserLabel, gbConst);
		pane.add(simChooserLabel);
		
		simChooser = new JComboBox(Student.values());
		gbConst.weightx = 3.0;
		gbConst.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(simChooser, gbConst);
		pane.add(simChooser);
		
		JLabel sideChooserLabel = new JLabel("Preferred side: ");
		gbConst.weightx= 1.0;
		gbConst.fill = GridBagConstraints.BOTH;
		gbConst.gridwidth = 1;
		gridBag.setConstraints(sideChooserLabel, gbConst);
		pane.add(sideChooserLabel);
		
		sideChooser = new JComboBox(Side.values());
		gbConst.weightx = 3.0;
		gbConst.gridwidth = GridBagConstraints.REMAINDER;
		gridBag.setConstraints(sideChooser, gbConst);
		pane.add(sideChooser);
		
		connectButton = new JButton("Connect");
		connectButton.addActionListener(this);
		gbConst.weightx = 2.0;
		gbConst.gridwidth = 2; //GridBagConstraints.RELATIVE;
		gridBag.setConstraints(connectButton, gbConst);
	    pane.add(connectButton);
		
	    goButton = new JButton("Go");
	    goButton.addActionListener(this);
		gridBag.setConstraints(goButton, gbConst);
	    pane.add(goButton);
	    
	    stopButton = new JButton("Disconnect");
	    stopButton.addActionListener(this);
		gbConst.weightx = 2.0;
		gbConst.gridwidth = 2;//GridBagConstraints.REMAINDER;
	    gridBag.setConstraints(stopButton, gbConst);
	    pane.add(stopButton);

	    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    setButtons();
	    setVisible(true);
		mUIReady = true; // must be last line of method 
		log.exiting("team.MainUI", "createAndShowUI");
	}
	
	/** Get a list of available view implementations.
	 * @return vector containing Students who have view implementations available.
	 */
	private Vector<Student> availableTeamImplementations() {
		Vector<Student> availTeams = new Vector<Student>();
		for (Student s : Student.values()) {
			try {
				Class.forName(teamName(s));
				availTeams.add(s);
			} catch (ClassNotFoundException e) { }
		}
		return availTeams;
	}
		
	/**
	 * Construct the team implementation name for a given student
	 * @param s the student
	 * @return the fully qualified team class name
	 */
	private String teamName(Student s) {
		return new String("team." + s.toString() + ".TeamMain");
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		createAndShowUI();
	}
	
	public void actionPerformed(ActionEvent e) {
		Object src = e.getSource();
		if (src == connectButton) {
			try {
				log.fine("Constructing team.");
				constructTeam();
				mNet = new game.net.SCORETeam();
				if (mNet.connect(simHost.getText(), (Student)simChooser.getSelectedItem(), mTeam.getTeam())) {
					mTeam.setSimulator(mNet);
				} else {
					log.info("Connection failed.");
					mTeam.stop();
					mTeam = null;
				}
			} catch (Exception ex) {
				log.warning("Exception from connect: " + ex.toString());
			}
		} else if (src == goButton) {
			log.fine("Starting game.");
			mTeam.start((Side)(sideChooser.getSelectedItem()));
		} else if (src == stopButton) {
			log.fine("Stoping team.");
			mTeam.stop();
			try {
				Reply repl = mNet.quit();
				if (repl.getType() == Reply.ReplyType.QUIT) {
					log.fine("Clean quit.");
				}
			} catch (Exception e1) {
				log.fine("Exception from quit: " + e1);
			}
			mTeam = null;
		}
		setButtons();
		
	}

	/* (non-Javadoc)
	 * @see team.ITeamObserver#statusChange()
	 */
	public void statusChange() {
		log.fine("Status changed to: " + mTeam.getStatus());
		setButtons();
		
	}
	
	/** 
	 * Set up the team to play the game.
	 * @throws InstantiationException
	 * @throws ClassNotFoundException no implementations available.
	 * @throws IllegalAccessException
	 */
	private void constructTeam() 
		throws InstantiationException, ClassNotFoundException, IllegalAccessException {
		
		Student v = (Student)teamChooser.getSelectedItem();
		if (v == null) throw new ClassNotFoundException();
		mTeam = (ITeam)Class.forName(teamName(v)).newInstance();
		mTeam.setStatusObserver(this);
	}
	
	/**
	 * Enable or disable buttons as appropriate for Team state.
	 */
	private void setButtons() {
		if (mTeam != null) {
			switch (mTeam.getStatus()) {
			case DONE:
			case INIT:
				connectButton.setEnabled(false);
				goButton.setEnabled(true);
				break;
				
			case RUNNING:
				connectButton.setEnabled(false);
				goButton.setEnabled(false);
				break;
			}
			stopButton.setEnabled(true);
		} else {
			connectButton.setEnabled(true);
			goButton.setEnabled(false);
			stopButton.setEnabled(false);		
		}
		
	}
	
	public void paint(Graphics g) {
		setButtons();
		super.paint(g);
	}
	
	/** Indicates if UI has been constructed */
	private boolean mUIReady;

	private JButton goButton;
	private JButton stopButton;
	private JButton connectButton;
	
	private JComboBox teamChooser;
	private JComboBox simChooser;
	private JTextField simHost;
	private JComboBox sideChooser;
	
	private ITeam mTeam;
	
	private game.net.SCORETeam mNet;
	
	/** Initial width of frame. */
	private static final int WIDTH = 410;
	
	/** Initial height of frame. */
	private static final int HEIGHT = 200;
	
	private static Logger log;
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		log = Logger.getLogger("Soccer.team");
		try {
			System.out.println("Log file will be written to: " + 
					System.getProperty("user.home"));
			Handler fh = new FileHandler("%h/soccer_team.log", true);
			fh.setFormatter(new SimpleFormatter());
			log.addHandler(fh);
			log.setLevel(Level.FINER); // could be set by UI
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		log.fine("Constructing new team.MainUI()");
		MainUI ui = new MainUI();

		try {
			while (!ui.uiIsReady()) Thread.sleep(500);
			
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			log.severe(e.toString());
		}

	}

}
